/* AUTOGENERATED FILE. DO NOT EDIT. */

/*=======Test Runner Used To Run Each Test Below=====*/
#define RUN_TEST(TestFunc, TestLineNum) \
{ \
  Unity.CurrentTestName = #TestFunc; \
  Unity.CurrentTestLineNumber = TestLineNum; \
  Unity.NumberOfTests++; \
  if (TEST_PROTECT()) \
  { \
      setUp(); \
      TestFunc(); \
  } \
  if (TEST_PROTECT()) \
  { \
    tearDown(); \
  } \
  UnityConcludeTest(); \
}

/*=======Automagically Detected Files To Include=====*/
#ifdef __WIN32__
#define UNITY_INCLUDE_SETUP_STUBS
#endif
#include "unity.h"
#ifndef UNITY_EXCLUDE_SETJMP_H
#include <setjmp.h>
#endif
#include <stdio.h>
#include "../../src/include/hashset.h"
#include "../../src/include/hstr_utils.h"
#include "../../src/include/hstr_history.h"
#include "../../src/include/hstr_favorites.h"
#include "../../src/include/hstr.h"
#include <string.h>
#include <regex.h>
#include <stdio.h>
#include <stdbool.h>
#include <getopt.h>

/*=======External Functions This Runner Calls=====*/
extern void setUp(void);
extern void tearDown(void);
extern void test_args(void);
extern void test_getopt(void);
extern void test_locate_char_in_string_overflow(void);
extern void test_favorites(void);
extern void test_hashset_blacklist();
extern void test_hashset_get_keys();
extern void test_regexp(void);
extern void test_help_long(void);
extern void test_help_short(void);
extern void test_string_elide();
extern void test_parse_history_line();


/*=======Suite Setup=====*/
static void suite_setup(void)
{
#if defined(UNITY_WEAK_ATTRIBUTE) || defined(UNITY_WEAK_PRAGMA)
  suiteSetUp();
#endif
}

/*=======Suite Teardown=====*/
static int suite_teardown(int num_failures)
{
#if defined(UNITY_WEAK_ATTRIBUTE) || defined(UNITY_WEAK_PRAGMA)
  return suiteTearDown(num_failures);
#else
  return num_failures;
#endif
}

/*=======Test Reset Option=====*/
void resetTest(void);
void resetTest(void)
{
  tearDown();
  setUp();
}


/*=======MAIN=====*/
int main(void)
{
  suite_setup();
  UnityBegin("../test/src/test.c");
  RUN_TEST(test_args, 49);
  RUN_TEST(test_getopt, 82);
  RUN_TEST(test_locate_char_in_string_overflow, 165);
  RUN_TEST(test_favorites, 176);
  RUN_TEST(test_hashset_blacklist, 200);
  RUN_TEST(test_hashset_get_keys, 215);
  RUN_TEST(test_regexp, 236);
  RUN_TEST(test_help_long, 276);
  RUN_TEST(test_help_short, 292);
  RUN_TEST(test_string_elide, 308);
  RUN_TEST(test_parse_history_line, 340);

  return suite_teardown(UnityEnd());
}
